﻿using System.IO;
using Infrastructure.Data;
using Infrastructure.Data.OliVerse;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Rewrite;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Newtonsoft.Json;
using Westwind.AspNetCore.LiveReload;

namespace Web
{
    public class Startup
    {
        public Startup(IConfiguration configuration, IWebHostEnvironment hostEnvironment)
        {
            Configuration = configuration;
            Env = hostEnvironment;
        }

        public IWebHostEnvironment Env { get; }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddLiveReload(config =>
            {
                config.FolderToMonitor = Path.GetFullPath(Path.Combine(Env.ContentRootPath, ".."));
            });
            
            services.AddDbContext<ExDbContext>(opt =>
                opt.UseSqlServer(Configuration.GetConnectionString("" /*TODO*/))
                    .AddInterceptors(new OutputCommandInterceptor()));
            services.AddRazorPages(options => { });
            services.AddControllersWithViews(options => { }); //(options => options.EnableEndpointRouting = false);

        

            //.AddJsonOptions(options => { options.JsonSerializerOptions.; }); 
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            else
            {
                // dont redirect if in development
                app.UseHttpsRedirection();
                app.UseExceptionHandler("/Error");

                // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
                app.UseHsts();
            }

            app.UseLiveReload();
            
            app.UseStaticFiles(new StaticFileOptions() {ServeUnknownFileTypes = true, });

            app.UseRouting();

//            app.UseRewriter(new RewriteOptions().AddRedirect("", "/"));

            // example of custom Endpoint-based routing examples:
            // https://github.com/dotnet/aspnetcore/issues/4221#issuecomment-488596903
            // https://github.com/dotnet/aspnetcore/blob/master/src/Mvc/test/WebSites/RoutingWebSite/StartupForDynamic.cs

            // old way of doing routing, requires options => options.EnableEndpointRouting = false
            // app.UseMvc(routes =>
            // {
            //     routes.MapRoute("default", "{controller=Home}");
            //     routes.MapSpaFallbackRoute("spa-fallback",
            //     new
            //     {
            //         controller = "Home",
            //         action = "Index"
            //     });
            // });
            //
            //*/


            app.UseEndpoints(endpoints =>
            {
                //endpoints.MapControllerRoute("default", "{controller=Home}/{action=Index}");

                // try and match the URL to razor pages first
                endpoints.MapRazorPages();

                // then try and map the URL to Controllers (including the API)
                //endpoints.MapControllerRoute("default", "/{controller}/");
                endpoints.MapControllers();

                // and if that doesnt work, send them the homepage, but leave their URI path intact as for handling by the SPA
                endpoints.MapFallbackToController("Index", "Home");
                
                //    endpoints.MapFallbackToPage("/");
                //endpoints.MapToSpaCliProxy("/");

                // Note: only use spacliproxy in development. 
                // Production should use "UseSpaStaticFiles()"
            }); //*/

            // app.UseEndpoints(endpoints =>
            // {
            //     endpoints.MapRazorPages();
            //     endpoints.MapControllers();
            // });
        }
    }
}